<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Block\Adminhtml;

/**
 * @inheritdoc
 */
class ButtonList extends \Magento\Backend\Block\Widget\Button\ButtonList
{

    /**
     *
     * @param \Magento\Backend\Block\Widget\Button\ItemFactory $itemFactory
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\UrlInterface $urlBuilder
     * @param \Magento\Framework\App\Request\Http $request
     */
    public function __construct(
        \Magento\Backend\Block\Widget\Button\ItemFactory $itemFactory,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Framework\App\Request\Http $request
    ) {
            $this->registry = $registry;
            $this->urlBuilder = $urlBuilder;
            $this->request = $request;

            parent::__construct($itemFactory);

            $this->init();
    }

    /**
     *
     * @return void
     */
    public function init()
    {
        $methodTarget = \Montonio\Hirepurchase\Model\Payment\ConfigProvider::CODE;
        $methodCode = $this->getOrder()
        ->getPayment()
        ->getMethodInstance()
        ->getCode() ?? false;

        if ($methodCode && $methodCode == $methodTarget) {
            $this->add('montonio_status_update_button', [
                'label' => __('Update Payment Status'),
                'onclick' => "setLocation('{$this->getStatusUpdateUrl()}')"
                ]);

            $this->updateData();
        }
    }

    /**
     *
     * @return void
     */
    public function updateData()
    {
        $info = $this->getOrder()->getPayment();
        $data = $info->getAdditionalInformation('gateway_response') ?? [];
        if (empty($data['status']) || $data['status'] !== 'signed') {
            if ($this->request->getParam('update_payment_status') === "1") {
                $info->getMethodInstance()
                ->validateStatusRequest(
                    ['payment' => $info->getMethodInstance()->getInfoInstance()]
                );
            }
        }
    }

    /**
     * Retrieve order model object
     *
     * @return \Magento\Sales\Model\Order
     */
    public function getOrder()
    {
        return $this->registry->registry('sales_order');
    }

    /**
     * Retrieve Order Identifier
     *
     * @return int
     */
    public function getOrderId()
    {
        return $this->getOrder() ? $this->getOrder()->getId() : null;
    }

    /**
     * Credit memo URL getter
     *
     * @return string
     */
    public function getStatusUpdateUrl()
    {
        return $this->getUrl('sales/order/view', [
            'order_id' => $this->getOrderId(),
            'update_payment_status' => 1
        ]);
    }

    /**
     * Generate url by route and parameters
     *
     * @param string $route
     * @param array $params
     * @return string
     */
    public function getUrl($route = '', $params = [])
    {
        return $this->urlBuilder->getUrl($route, $params);
    }
}
