<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

declare(strict_types=1);

namespace Montonio\Hirepurchase\Block\Frontend\LayoutProcessors;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\View\Asset\Repository as AssetRepository;
use Montonio\Hirepurchase\Gateway\Payments\Config\Config;

/**
 * {@inheritdoc}
 */
class PaymentsProcessor implements \Magento\Checkout\Block\Checkout\LayoutProcessorInterface
{
    private const DEFAULT_SERVICE_IMAGE_LOCATION = 'Montonio_Hirepurchase::images/montonio_darkblue_medium.png';

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var AssetRepository
     */
    private $assetRepository;

    /**
     * @param ScopeConfigInterface $scopeConfig
     * @param Config $config
     * @param AssetRepository $assetRepository
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Config $config,
        AssetRepository $assetRepository
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->config = $config;
        $this->assetRepository = $assetRepository;
    }

    /**
     * {@inheritdoc}
     */
    public function process($jsLayout)
    {
        $enabled = $this->config->isActive();
        if (!$enabled) {
            return $jsLayout;
        }

        $methodCode = $this->config->getCode();

        $layoutPath = $jsLayout['components']['checkout']
            ['children']['steps']
            ['children']['billing-step']
            ['children']['payment']
            ['children']['renders']
            ['children'] ?? false;

        if ($layoutPath) {
            $paymentMethods = [];

            $availableMethods = $this->config->getAvailableMethods();

            //if grouped we ignore other methods
            $isGrouped = $this->config->getValue('grouped');
            if ($isGrouped) {
                try {
                    $defaultImage = $this->assetRepository->getUrl(self::DEFAULT_SERVICE_IMAGE_LOCATION);
                } catch (\Throwable $exception) {
                    $defaultImage = null;
                }

                $availableMethods = ['grouped' => ['logo_url' => $defaultImage]];
            }

            foreach ($availableMethods as $code => $data) {
                $subMethodCode = $methodCode. '_' .$code;
                $title = (isset($data['method_title'])) ? $data['method_title'] : false;
                $description = (isset($data['method_desc'])) ?
                    $data['method_desc'] : $this->config->getServiceDescription();
                $logoPath = (isset($data['logo_url'])) ? $data['logo_url'] : $this->config->getServiceLogoPath();
                $paymentMethods[$subMethodCode] = [
                    'component' => 'Montonio_Hirepurchase/js/view/payment/montonio',
                    'cnf' => [
                        'methodCode' => $subMethodCode,
                        'title' => $title,
                        'serviceDescription' => $description,
                        'serviceLogoPath' => $logoPath,
                        'titleDisplayOption' => $this->config->getTitleDisplayOption()
                    ]
                ];
            }
            $paymentMethods = array_merge($layoutPath, $paymentMethods);

            $jsLayout['components']['checkout']
                ['children']['steps']
                ['children']['billing-step']
                ['children']['payment']
                ['children']['renders']
                ['children'] = $paymentMethods;
        }
        return $jsLayout;
    }
}
