<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);
namespace Montonio\Hirepurchase\Gateway\Hirepurchase\Config;

/**
 * @inheritdoc
 */
class Config extends \Magento\Payment\Gateway\Config\Config
{

    /**
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Escaper $escaper
     * @param string $methodCode
     * @param string $pathPattern
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Escaper $escaper,
        $methodCode = null,
        $pathPattern = self::DEFAULT_PATH_PATTERN
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->escaper = $escaper;

        //if we DI this class directly to our other components we need this to be initiated
        //in all other case it is initiated trough di.xml
        if (!$methodCode) {
            $methodCode = \Montonio\Hirepurchase\Model\Payment\ConfigProvider::CODE;
        }

        $this->methodCode = $methodCode;

        parent::__construct($scopeConfig, $methodCode, $pathPattern);
    }

    /**
     * @return bool
     */
    public function isActive($storeId = null)
    {
        return $this->getValue('active', $storeId);
    }

    /**
     *
     * @return string
     */
    public function getCode()
    {
        return $this->methodCode;
    }

    /**
     *
     * @return string
     */
    public function getServiceDescription()
    {
        return $this->escaper->escapeHtml($this->getValue('service_description'));
    }

    /**
     *
     * @return string
     */
    public function getServiceLogoPath()
    {
        return $this->escaper->escapeHtml($this->getConfig('service_logo_path'));
    }

    /**
     * Retrieve assoc array of checkout configuration
     *
     * @return array
     */
    public function getConfig()
    {
        //we do not use this and return empty array
        return [];
    }

    /**
     *
     * @param mixed $data
     * @param string $key
     * @return string
     */
    public function generateSignature($data, $key, $jsonEncode = true)
    {
        if ($jsonEncode) {
            $data = json_encode($data);
        }

        return hash_hmac('sha256', $data, $key);
    }
}
