<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Gateway\Hirepurchase\Http\Client;

use Magento\Payment\Gateway\Http\ClientInterface;
use Magento\Payment\Gateway\Http\TransferInterface;
use Magento\Framework\HTTP\ClientFactory;
use Magento\Payment\Gateway\Http\ClientException;
use Magento\Payment\Model\Method\Logger;

/**
 * @inheritdoc
 */
class PreflightTransfer implements ClientInterface
{

    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var Curl
     */
    private $curl;

    /**
     *
     * @param Logger $logger
     * @param ClientFactory $httpClientFactory
     */
    public function __construct(
        Logger $logger,
        ClientFactory $httpClientFactory
    ) {
        $this->logger = $logger;
        $this->httpClientFactory = $httpClientFactory;
    }

    /**
     * Places request to gateway. Returns result as ENV array
     *
     * @param TransferInterface $transferObject
     * @return array
     */
    public function placeRequest(TransferInterface $transferObject)
    {

        try {
            $client = $this->getCurlClient();
            $client->setHeaders($transferObject->getHeaders());
            if ($transferObject->getMethod() === 'GET') {
                $client->get($transferObject->getUri());
            } else {
                $client->post($transferObject->getUri(), $transferObject->getBody());
            }
            $responseBody = $client->getBody();
            $response = json_decode($client->getBody(), true);
        } catch (\Exception $e) {
            $this->logger->critical($e);
            throw new ClientException(__('Something went wrong in the payment gateway.'));
        }

        return $response;
    }

    /**
     * @return Curl
     */
    public function getCurlClient()
    {
        return $this->httpClientFactory->create();
    }
}
