<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\Handler;

use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Api\Data\InvoiceInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\InvoiceManagementInterface;
use Magento\Sales\Model\Order\Invoice;
use Montonio\Hirepurchase\Gateway\Payments\Config\Config as PaymentsConfig;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\ResponseHandlerInterface;

/**
 * Handler responsible for creating invoice object and sending invoice e-mail.
 */
class InvoiceHandler implements ResponseHandlerInterface
{
    /**
     * @var InvoiceManagementInterface
     */
    private $invoiceManagement;

    /**
     * @var  PaymentsConfig
     */
    private $paymentsConfig;

    /**
     *
     * @param InvoiceManagementInterface $invoiceManagement
     * @param PaymentsConfig $paymentsConfig
     */
    public function __construct(
        InvoiceManagementInterface $invoiceManagement,
        PaymentsConfig $paymentsConfig
    ) {
        $this->invoiceManagement = $invoiceManagement;
        $this->paymentsConfig = $paymentsConfig;
    }

    /**
     * @inheritdoc
     */
    public function handle(OrderInterface $order, array $data): void
    {

        $invoice = $this->prepareInvoice($order);
    }

    /**
     * Register order invoice.
     *
     * @param OrderInterface $order
     *
     * @return Invoice|null
     *
     * @throws LocalizedException
     */
    private function prepareInvoice(OrderInterface $order): ?Invoice
    {
        $controlInvoice = (bool)$this->paymentsConfig->getValue('create_invoice') ?? false;
        if (!$order->canInvoice() ||
            !$controlInvoice
        ) {
            return null;
        }

        /** @var Invoice $invoice */
        $invoice = $this->invoiceManagement->prepareInvoice($order);
        $invoice->register();
        $invoice->addComment(__('Invoice generated together with order confirmation from payment gateway'), false, true);
        $invoice->save();

        $controlInvoiceEmail = (bool)$this->paymentsConfig->getValue('invoice_email_after') ?? false;
        if ($controlInvoiceEmail) {
            $result = (bool)$this->invoiceManagement->notify($invoice->getEntityId());
            if ($result === true) {
                $invoice->addComment(__('Invoice e-mail sent to customer'), false, true);
            }
        }
        $order->addRelatedObject($invoice);

        return $invoice;
    }
}
