<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Split\Validator;

use \Magento\Payment\Gateway\Validator\AbstractValidator;
use \Magento\Payment\Gateway\Validator\ResultInterfaceFactory;
use Montonio\Hirepurchase\Gateway\Split\Config\Config;
use Montonio\Hirepurchase\Model\Payment\SplitConfigProvider;

/**
 * @inheritdoc
 */
class CountryValidator extends AbstractValidator
{
    /**
     * @var Config
     */
    private $config;

    /**
     * @inheritdoc
     */
    public function __construct(
        ResultInterfaceFactory $resultFactory,
        Config $config
    ) {
        $this->config = $config;
        parent::__construct($resultFactory);
    }

    /**
     * @inheritdoc
     */
    public function validate(array $validationSubject)
    {

        $isValid = true;
        $storeId = $validationSubject['storeId'];

        $allowSpecific = (int)$this->config->getValue('allowspecific', $storeId);
        $specificCountry = $this->config->getValue('specificcountry', $storeId);

        if ($allowSpecific === 1) {
            $availableCountries = explode(',', $specificCountry);
            if (!in_array($validationSubject['country'], $availableCountries)) {
                $isValid =  false;
            }
        }

        $methodConfigs = $this->config->getAvailableMethods();

        foreach ($methodConfigs as $code => $value) {
            $subCode = $this->config->getCode() . '_' . $code;

            if ($subCode == $validationSubject['methodCode']) {
                if ($validationSubject['country'] == $value['method_country'] ||
                    $value['method_country'] == ''
                ) {
                    $isValid = true;
                    break;
                }
            }

            $isValid =  false;
        }

        return $this->createResult($isValid);
    }
}
